import "dart:convert";
import "dart:mirrors";
import "../helpers/reflection_helper.dart";

class Export {
  const Export();
}

abstract class Entity {
  factory Entity.fromJson(Type entityType, String json) {
    final Map jsonMap = JSON.decode(json);
    final typeMirror = reflectClass(entityType);
    final obj = typeMirror.newInstance(new Symbol(""), const []);
    jsonMap.forEach((k, v) {
      final fieldName = new Symbol(k);
      final fieldTypeKey =
          typeMirror.declarations.keys.firstWhere((key) => key == fieldName);
      final fieldTypeMirror =
          (typeMirror.declarations[fieldTypeKey] as VariableMirror).type;
      obj.setField(fieldName, _deserialize(fieldTypeMirror, v));
    });
    return obj.reflectee;
  }

  String toJson() {
    final instanceMirror = reflect(this);
    final classMirror = instanceMirror.type;
    final map = {};
    for (VariableMirror field
        in classMirror.declarations.values.where((t) => t is VariableMirror)) {
      if (hasTag(field, Export)) {
        final key = MirrorSystem.getName(field.simpleName);
        map[key] =
            _serialize(field.type, instanceMirror.getField(field.simpleName));
      }
    }
    return JSON.encode(map);
  }

  static dynamic _deserialize(TypeMirror instanceTypeMirror, dynamic v) {
    if (instanceTypeMirror.isAssignableTo(reflectType(Entity))) {
      return _fromJsonReflect(instanceTypeMirror.reflectedType, v);
    } else if (instanceTypeMirror.isAssignableTo(reflectType(List))) {
      return v
          .map((x) => _deserialize(instanceTypeMirror.typeArguments.first, x))
          .toList();
    } else if (reflectClass(instanceTypeMirror.reflectedType).isEnum) {
      return _enumFromInt(instanceTypeMirror.reflectedType, v);
    } else {
      return v;
    }
  }

  dynamic _serialize(TypeMirror instanceType, InstanceMirror instanceMirror) {
    if (instanceType.isAssignableTo(reflectType(Entity))) {
      return _toJsonReflect(instanceMirror);
    } else if (instanceType.isAssignableTo(reflectType(List))) {
      return instanceMirror.reflectee
          .map((e) => _serialize(instanceType.typeArguments.first, reflect(e)))
          .toList();
    } else if (reflectClass(instanceType.reflectedType).isEnum) {
      return _enumToInt(instanceType.reflectedType, instanceMirror.reflectee);
    } else {
      return instanceMirror.reflectee;
    }
  }

  static dynamic _enumFromInt(Type t, int value) {
    var values =
        (reflectType(t) as ClassMirror).getField(#values).reflectee as List;
    if (value == -1 || value >= values.length) {
      return null;
    }
    return values[value];
  }

  int _enumToInt(Type t, dynamic value) =>
      ((reflectType(t) as ClassMirror).getField(#values).reflectee as List)
          .indexOf(value);

  static dynamic _fromJsonReflect(Type type, String map) =>
      reflectClass(type).newInstance(new Symbol("fromJson"), [map]).reflectee;

  static String _toJsonReflect(InstanceMirror instanceMirror) =>
      instanceMirror.invoke(new Symbol("toJson"), const []).reflectee;
}
