import "package:test/test.dart";
import "../lib/src/models/entity.dart";

enum CareerStage { Junior, Middle, Senior }

class Employee extends Object with Entity {
  @Export()
  String name;
  @Export()
  int age;
  @Export()
  CareerStage stage;

  Employee();
  factory Employee.fromJson(String json) => new Entity.fromJson(Employee, json);

  @override
  String toString() => "$name $age $stage";
}

class Office extends Object with Entity {
  @Export()
  List<Employee> employees;

  Office();
  factory Office.fromJson(String json) => new Entity.fromJson(Office, json);

  @override
  String toString() => "employees : ${employees.join(", ")}";
}

class ContactsTable extends Object with Entity {
  @Export()
  List<List<Employee>> contactsLists;

  ContactsTable();
  factory ContactsTable.fromJson(String json) =>
      new Entity.fromJson(ContactsTable, json);
}

void main() {
  group("toJson tests", () {
    Employee employee;
    Office office;
    ContactsTable table;
    setUp(() {
      employee = new Employee()
        ..name = "John Smith"
        ..age = 42
        ..stage = CareerStage.Middle;
      office = new Office()..employees = new List<Employee>();
      office.employees.add(employee);
      office.employees.add(employee);
      office.employees.add(employee);
      table = new ContactsTable()..contactsLists = new List<List<Employee>>();
      table.contactsLists.add(office.employees);
      table.contactsLists.add(office.employees);
      table.contactsLists.add(office.employees);
    });

    test("Entity can serialize PODOs", () {
      expect(employee.toString(),
          equals(new Employee.fromJson(employee.toJson()).toString()));
    });

    test("Entity can serialize null enums", () {
      employee.stage = null;
      expect(employee.toString(),
          equals(new Employee.fromJson(employee.toJson()).toString()));
    });

    test("Entity can serialize PODO lists", () {
      expect(office.toString(),
          equals(new Office.fromJson(office.toJson()).toString()));
    });

    test("Entity can serialize list of lists of PODOs", () {
      expect(table.toString(),
          equals(new ContactsTable.fromJson(table.toJson()).toString()));
    });
  });
}
